﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;




namespace Hitager
{


    public partial class Hitag2 : UserControl
    {

        static Hitag_App.CryptoInterface cryptoInterface = new Hitag_App.CryptoInterface();

        public Hitag2()
        {
            InitializeComponent();
        }

        TextBox debugTextBox;
        private void handleDebug(String text)
        {
            debugTextBox.AppendText(text + Environment.NewLine);
        }
        public void setDebug(ref TextBox textbox)
        {
            debugTextBox = textbox;
        }

        private PortHandler portHandler;
        public void SetPortHandler(ref PortHandler handler)
        {
            portHandler = handler;
        }

        private void FactoryIsk_Click(object sender, EventArgs e)
        {
            TbISK.Text = "4D494B524F4E";
        }

        private void ReadButton_Click(object sender, EventArgs e)
        {
            ReadButton.Enabled = false;
            P0.Text = "";
            P1.Text = "";
            P2.Text = "";
            P3.Text = "";
            P4.Text = "";
            P5.Text = "";
            P6.Text = "";
            P7.Text = "";

            byte[] haltCard = { 0x1f, 0x00 };

            if (!handleISK())
            {
                ReadButton.Enabled = true;
                return;
            }
            else
            {
                portHandler.portWR("o");

                String cardID = portHandler.portWR("i05C0");
                if (cardID.Equals("TIMEOUT"))
                {
                    handleDebug("TIMEOUT");

                }
                else if (cardID.Equals(""))
                {
                    handleDebug("ERROR ID");
                    handleDebug("Please retry reading!");
                }
                else
                {

                    String key = cryptoInterface.GetSignature(cardID);
                    handleDebug(key);
                    String page3c = portHandler.portWR("i40" + key);
                    handleDebug(page3c);
                    if (page3c.Equals("TIMEOUT"))
                    {
                        handleDebug("TIMEOUT");
                    }
                    else
                    {
                        String page3 = cryptoInterface.CryptString(page3c, 32);
                        if (page3.Equals("ERROR"))
                        {
                            handleDebug("CRYPT ERROR");
                        }
                        else
                        {
                            page3 = page3.Substring(0, 8);
                            handleDebug("P3:" + page3);

                            P0.Text = cardID.Substring(0, 8);

                            if (page3c.Equals("") || page3c.Equals("0AAAAAAA"))
                            {
                                P3.Text = "Wrong key";

                            }
                            else
                            {
                                P1.Text = ReadPage(1);
                                P2.Text = ReadPage(2);
                                P3.Text = page3;
                                P4.Text = ReadPage(4);
                                P5.Text = ReadPage(5);
                                P6.Text = ReadPage(6);
                                P7.Text = ReadPage(7);
                            }
                        }
                    }
                }

                portHandler.portWR("f");
                ReadButton.Enabled = true;
            }
        }
        private bool handleISK()
        {
            String sisk = TbISK.Text;
            if (sisk.Length != 12)
            {
                handleDebug("Incorrect ISK");
                return false;
            }
            cryptoInterface.setISK(sisk); ;
            return true;
        }
        private String ReadPage(int page)
        {
            int cmd = 0xC000;
            cmd |= (page & 0x7) << 11 | (((~page) & 0x7) << 6);
            byte[] pageread = { (byte)(0xff & (cmd >> 8)), (byte)(0xff & cmd) };
            String pagereadc = cryptoInterface.Crypt(pageread, 10);
            String pagec = portHandler.portWR("i0a" + pagereadc);
            if (pagec.Length == 0)
                return "ERROR";
            handleDebug(pagec );
            if (pagec.Length < 8)
                return "ERROR";
            String pageText = cryptoInterface.CryptString(pagec, 32);
            pageText = pageText.Substring(0, 8);
            handleDebug("P" + page + ": " + pageText);

            return pageText;
        }

        private String WritePage(int page, String content)
        {

            handleISK();
            if (content.Length != 8)
            {
                handleDebug("INCORRECT CONTENT");
                return "ERROR";
            }

            portHandler.portWR("o");

            String cardID = portHandler.portWR("i05C0");
            if (cardID.Equals("TIMEOUT"))
            {
                handleDebug("TIMEOUT");
                portHandler.portWR("f");
                return "ERROR";
            }
            if (cardID.Equals(""))
            {
                handleDebug("ERROR ID");
                portHandler.portWR("f");
                return "ERROR";
            }

            String key = cryptoInterface.GetSignature(cardID);
            handleDebug(key);
            String page3c = portHandler.portWR("i40" + key);
            handleDebug(page3c);
            if (page3c.Equals("TIMEOUT"))
            {
                portHandler.portWR("f");
                return "ERROR";

            }
            String page3 = cryptoInterface.CryptString(page3c, 32);
            handleDebug("P3:" + page3);

            int cmd = 0x8200;
            cmd |= (page & 0x7) << 11 | (((~page) & 0x7) << 6);
            byte[] pagewrite = { (byte)(0xff & (cmd >> 8)), (byte)(0xff & cmd) };
            String pagewriteS = pagewrite[0].ToString("X2") + pagewrite[1].ToString("X2");
            String pagewritec = cryptoInterface.Crypt(pagewrite, 10);
            String pagewreplyc = portHandler.portWR("i0a" + pagewritec);
            if (pagewreplyc.Length == 0)
            {
                portHandler.portWR("f");
                return "ERROR";
            }
            handleDebug(pagewreplyc);
            String pagewreply = cryptoInterface.CryptString(pagewreplyc + "00", 10);
            if (pagewreply.StartsWith(pagewriteS.Substring(0, 2)))
            {
                String contentc = cryptoInterface.CryptString(content);
                pagewreplyc = portHandler.portWR("i20" + contentc);

                portHandler.portWR("f");

                return "OK";
            }
            else
            {
                portHandler.portWR("f");
                return "ERROR";

            }
        }

        private void Bw0_Click(object sender, EventArgs e)
        {
            //if(P0.Text.Length==8)
            WritePage(0, P0.Text);
        }

        private void Bw1_Click(object sender, EventArgs e)
        {
            WritePage(1, P1.Text);
        }

        private void Bw2_Click(object sender, EventArgs e)
        {
            WritePage(2, P2.Text);
        }

        private void Bw3_Click(object sender, EventArgs e)
        {
            WritePage(3, P3.Text);
        }

        private void Bw4_Click(object sender, EventArgs e)
        {
            WritePage(4, P4.Text);
        }

        private void Bw5_Click(object sender, EventArgs e)
        {
            WritePage(5, P5.Text);
        }

        private void Bw6_Click(object sender, EventArgs e)
        {
            WritePage(6, P6.Text);
        }

        private void Bw7_Click(object sender, EventArgs e)
        {
            WritePage(7, P7.Text);
        }

        private void TbISK_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsControl(e.KeyChar) && !e.KeyChar.IsHex())
            {
                
                e.Handled = true;
            }
            else
            {
                e.KeyChar = char.ToUpper(e.KeyChar);
            }
        }

        private void P0_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!char.IsControl(e.KeyChar) && !e.KeyChar.IsHex())
            {

                e.Handled = true;
            }
            else if(!char.IsControl(e.KeyChar))
            {
                e.KeyChar = char.ToUpper(e.KeyChar);
            }
        }

        private void cardIskToFactory_Click(object sender, EventArgs e)
        {
            string newIsk = tbNewIsk.Text;
            string oldIsk = TbISK.Text;
            string oldIskP1 = P1.Text;
            string oldIskP2 = P2.Text;
            if (oldIsk.Length != 12 || newIsk.Length != 12 || oldIskP1.Length!=8 || oldIskP2.Length!=8 || !oldIsk.StartsWith(oldIskP1) || !oldIsk.Substring(8,4).Equals(oldIskP2.Substring(4,4)))
            {
                handleDebug("Incorrect ISK settings");
                return;
            }
            string ret = WritePage(1, newIsk.Substring(0,8));
            if(ret.Equals("ERROR"))
            {
                handleDebug("Error writing P1");
                return;
            }
            TbISK.Text = newIsk.Substring(0, 8) + oldIsk.Substring(8, 4);
            ret = WritePage(2, oldIskP2.Substring(0,4) + newIsk.Substring(8, 4));
            if (ret.Equals("ERROR"))
            {
                handleDebug("Error writing P2");
                return;
            }
            TbISK.Text = newIsk;

        }

        private void FactoryIskSet_Click(object sender, EventArgs e)
        {
            tbNewIsk.Text = "4D494B524F4E";
        }
    }
}

public static class Extensions
{
    public static bool IsHex(this char c)
    {
        return (c >= '0' && c <= '9') ||
                 (c >= 'a' && c <= 'f') ||
                 (c >= 'A' && c <= 'F');
    }
}